#include "slim.h"
#include <vector>
#include <mutex>
#include <algorithm>  // For std::remove_if
#include <string>
#include <iostream>
#include <VersionHelpers.h>
#include <Psapi.h>
#include <intrin.h>
#include <sstream>

// Forward declarations for helper functions
std::string GetThreadIdString();
VOID ShowMessageWithThreadId(const char* title, const char* message);

// Globals
PFN_NT_QUEUE_APC_THREAD NtQueueApcThread = nullptr;
PFN_NT_TEST_ALERT NtTestAlert = nullptr;
PFN_NT_ALERT_THREAD_BY_THREAD_ID NtAlertThreadByThreadId = nullptr;
PFN_NT_WAIT_FOR_ALERT_BY_THREAD_ID NtWaitForAlertByThreadId = nullptr;
PFN_ZW_YIELD_EXECUTION ZwYieldExecution = nullptr;
PFN_RTL_DISPATCH_EXCEPTION RtlDispatchException = nullptr;

// Thread context list
std::vector<THREAD_CONTEXT> g_ThreadContexts;
THREAD_CONTEXT g_WorkerContext = {0};
HANDLE g_WorkerThread = NULL;
HANDLE g_WorkerEvent = NULL;

// Synchronization
std::mutex g_ThreadContextMutex;

// VEH calling pattern
PVOID g_VehCallingPattern = NULL;

// Add this function declaration at the top with other function declarations
NTSTATUS NTAPI TestAlertWrapper();

// Initialize the SlimBloc system
BOOL InitializeSlimBloc() {
    // Load NT functions from ntdll.dll
    HMODULE ntdll = GetModuleHandleA("ntdll.dll");
    if (!ntdll) return FALSE;

    NtQueueApcThread = (PFN_NT_QUEUE_APC_THREAD)GetProcAddress(ntdll, "NtQueueApcThread");
    NtTestAlert = (PFN_NT_TEST_ALERT)GetProcAddress(ntdll, "NtTestAlert");
    ZwYieldExecution = (PFN_ZW_YIELD_EXECUTION)GetProcAddress(ntdll, "ZwYieldExecution");
    RtlDispatchException = (PFN_RTL_DISPATCH_EXCEPTION)GetProcAddress(ntdll, "RtlDispatchException");

    // Load version-dependent functions
    if (IsWindows10OrGreater()) {
        NtAlertThreadByThreadId = (PFN_NT_ALERT_THREAD_BY_THREAD_ID)GetProcAddress(ntdll, "NtAlertThreadByThreadId");
        NtWaitForAlertByThreadId = (PFN_NT_WAIT_FOR_ALERT_BY_THREAD_ID)GetProcAddress(ntdll, "NtWaitForAlertByThreadId");
        LogMessage("Using Windows 10+ thread alert functions");
    } else {
        LogMessage("Using legacy thread synchronization for pre-Windows 10");
    }

    if (!NtQueueApcThread || !NtTestAlert || !ZwYieldExecution) {
        LogMessage("Failed to load required NT functions");
        return FALSE;
    }

    // Create synchronization event for worker thread
    g_WorkerEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (!g_WorkerEvent) {
        LogMessage("Failed to create worker event");
        return FALSE;
    }

    // Allocate worker thread context
    g_WorkerContext.IsWorker = TRUE;
    g_WorkerContext.SlimLock = (PSLIM_LOCK)VirtualAlloc(NULL, sizeof(SLIM_LOCK), MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
    if (!g_WorkerContext.SlimLock) {
        LogMessage("Failed to allocate worker SlimLock");
        return FALSE;
    }

    // Initialize worker Slim lock
    ZeroMemory(g_WorkerContext.SlimLock, sizeof(SLIM_LOCK));
    g_WorkerContext.SlimLock->WaitersList.Flink = &g_WorkerContext.SlimLock->WaitersList;
    g_WorkerContext.SlimLock->WaitersList.Blink = &g_WorkerContext.SlimLock->WaitersList;

    // Create worker thread
    LogMessage("Creating worker thread");
    g_WorkerThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)SmStartup, NULL, 0, (LPDWORD)&g_WorkerContext.ThreadId);
    if (!g_WorkerThread) {
        LogMessage("Failed to create worker thread");
        return FALSE;
    }

    g_WorkerContext.ThreadHandle = g_WorkerThread;
    DWORD workerThreadId = GetThreadId(g_WorkerThread);
    LogMessage("Worker thread created with ID: 0x%X", workerThreadId);

    // Wait for worker to initialize
    LogMessage("Waiting for worker thread initialization");
    Sleep(500);

    return TRUE;
}

// Queue a Slim APC to a thread
VOID SmAlertThread(HANDLE ThreadHandle) {
    if (!NtQueueApcThread) {
        LogMessage("NtQueueApcThread function not available");
        return;
    }

    if (!ThreadHandle || ThreadHandle == INVALID_HANDLE_VALUE) {
        LogMessage("Invalid thread handle provided to SmAlertThread");
        return;
    }

    DWORD threadId = GetThreadId(ThreadHandle);
    if (threadId == 0) {
        LogMessage("Failed to get thread ID for handle: error %d", GetLastError());
        return;
    }

    LogMessage("Setting up APC for thread 0x%X", threadId);

    // Lock for thread safety when searching/modifying the vector
    std::lock_guard<std::mutex> lock(g_ThreadContextMutex);

    // Check if this thread already has a context
    for (auto& ctx : g_ThreadContexts) {
        if (ctx.ThreadId == (HANDLE)(ULONG_PTR)threadId) {
            LogMessage("Thread already has a context, updating");

            // Check if the SlimLock is still valid before updating
            if (ctx.SlimLock) {
                MEMORY_BASIC_INFORMATION memInfo;
                if (VirtualQuery(ctx.SlimLock, &memInfo, sizeof(memInfo)) == 0 ||
                    memInfo.State != MEM_COMMIT) {
                    LogMessage("Existing SlimLock is invalid, allocating new one");
                    VirtualFree(ctx.SlimLock, 0, MEM_RELEASE);
                    ctx.SlimLock = NULL;
                }
            }

            // Allocate a new SlimLock if needed
            if (ctx.SlimLock == NULL) {
                ctx.SlimLock = (PSLIM_LOCK)VirtualAlloc(NULL, sizeof(SLIM_LOCK), MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
                if (!ctx.SlimLock) {
                    LogMessage("Failed to allocate SlimLock for thread");
                    return;
                }

                // Initialize Slim lock
                ZeroMemory(ctx.SlimLock, sizeof(SLIM_LOCK));
                ctx.SlimLock->WaitersList.Flink = &ctx.SlimLock->WaitersList;
                ctx.SlimLock->WaitersList.Blink = &ctx.SlimLock->WaitersList;
            }

            // Queue the APC
            NTSTATUS status = NtQueueApcThread(
                ThreadHandle,
                (PPS_APC_ROUTINE)RtlReleaseSRWLockExclusive,
                ctx.SlimLock,
                (PVOID)ZwYieldExecution,
                (PVOID)SmApc
            );

            LogMessage("APC queued with status 0x%X", status);

            // Signal the worker thread to start monitoring
            SmSignalEvent(g_WorkerContext.ThreadId);
            return;
        }
    }

    // Create a thread context and add it to our list
    THREAD_CONTEXT threadCtx = {0};

    // Create a TLS Slim structure
    threadCtx.SlimLock = (PSLIM_LOCK)VirtualAlloc(NULL, sizeof(SLIM_LOCK), MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
    if (!threadCtx.SlimLock) {
        LogMessage("Failed to allocate SlimLock for thread");
        return;
    }

    // Initialize Slim lock
    ZeroMemory(threadCtx.SlimLock, sizeof(SLIM_LOCK));
    threadCtx.SlimLock->WaitersList.Flink = &threadCtx.SlimLock->WaitersList;
    threadCtx.SlimLock->WaitersList.Blink = &threadCtx.SlimLock->WaitersList;

    // Get thread ID and store handle
    threadCtx.ThreadId = (HANDLE)(ULONG_PTR)threadId;
    DuplicateHandle(GetCurrentProcess(), ThreadHandle, GetCurrentProcess(),
                    &threadCtx.ThreadHandle, 0, FALSE, DUPLICATE_SAME_ACCESS);

    if (!threadCtx.ThreadHandle) {
        LogMessage("Failed to duplicate thread handle: error %d", GetLastError());
        threadCtx.ThreadHandle = ThreadHandle; // Use the original handle
    }

    // Add new thread context to the list
    g_ThreadContexts.push_back(threadCtx);
    LogMessage("Added new thread context to list, total contexts: %d", g_ThreadContexts.size());

    // Queue the APC
    NTSTATUS status = NtQueueApcThread(
        ThreadHandle,
        (PPS_APC_ROUTINE)RtlReleaseSRWLockExclusive,
        threadCtx.SlimLock,
        (PVOID)ZwYieldExecution,
        (PVOID)SmApc
    );

    LogMessage("APC queued with status 0x%X", status);

    // Signal the worker thread to start monitoring
    SmSignalEvent(g_WorkerContext.ThreadId);
}

// Signal a thread via alert or event
VOID SmSignalEvent(HANDLE ThreadId) {
    if (NtAlertThreadByThreadId && ThreadId) {
        LogMessage("Signaling thread 0x%X via NtAlertThreadByThreadId", ThreadId);
        NtAlertThreadByThreadId(ThreadId);
    } else if (g_WorkerEvent) {
        LogMessage("Signaling thread via event");
        SetEvent(g_WorkerEvent);
    }
}

// Replacement for RtlReleaseSRWLockExclusive that will be called by the APC
VOID RtlReleaseSRWLockExclusive(PSLIM_LOCK SlimLock) {
    // Validate pointer before using it
    if (!SlimLock) {
        LogMessage("ERROR: SlimLock pointer is NULL in RtlReleaseSRWLockExclusive");
        return;
    }

    // Additional validation to check if memory is accessible
    MEMORY_BASIC_INFORMATION memInfo;
    if (VirtualQuery(SlimLock, &memInfo, sizeof(memInfo)) == 0) {
        LogMessage("ERROR: SlimLock pointer memory cannot be queried");
        return;
    }

    // Check if memory is committed and has proper access rights
    if (memInfo.State != MEM_COMMIT ||
        !(memInfo.Protect & (PAGE_READWRITE | PAGE_EXECUTE_READWRITE))) {
        LogMessage("ERROR: SlimLock memory is not properly allocated or accessible: State=%d, Protect=%d",
                   memInfo.State, memInfo.Protect);
        return;
    }

    std::string threadId = GetThreadIdString();
    std::cout << "\n*** CFG BYPASS TRIGGERED - Entered RtlReleaseSRWLockExclusive ***\n" << std::endl;
    LogMessage("Entered RtlReleaseSRWLockExclusive replacement - Thread ID: %s", threadId.c_str());

    // Show MessageBox to demonstrate SRW lock execution
    std::cout << "Showing MessageBox for SRW Lock Handler" << std::endl;
    ShowMessageWithThreadId("SlimBloc SRW Lock", "Entered SRW Lock Handler\nStarting spin loop");

    try {
        // Signal the lock - wrapped in try/catch to prevent crashes
        SlimLock->Ptr = (PVOID)1; // Simple signaling for demo
        SlimLock->OwnerThread = (PVOID)(ULONG_PTR)GetCurrentThreadId();  // Fixed potentially unsafe cast for x64

        // Signal the worker thread to process our lock
        SmSignalEvent(g_WorkerContext.ThreadId);

        LogMessage("Entering spin loop, waiting for worker thread to route us");

        // Infinite loop waiting for the worker to route us
        volatile LONG spinCount = 0;
        while (SlimLock && SlimLock->Ptr) {
            // This is the spin loop that will be interrupted by worker
            if ((spinCount++ % 1000) == 0) {
                Sleep(0);  // Yield time slice but remain alertable
            }

            // Safety check - don't spin forever
            if (spinCount > 10000000) {
                LogMessage("Spin loop timeout - breaking out");
                break;
            }
        }

        LogMessage("Exiting spin loop, control will transfer to redirection target");
        ShowMessageWithThreadId("SlimBloc SRW Lock", "Exiting spin loop\nControl transferring to ZwYieldExecution");
    }
    catch (const std::exception& e) {
        LogMessage("Exception in RtlReleaseSRWLockExclusive: %s", e.what());
    }
    catch (...) {
        LogMessage("Unknown exception in RtlReleaseSRWLockExclusive");
    }

    // When loop exits, we'll return to our redirect target (ZwYieldExecution)
}

// Apc handler that's executed after the thread is routed
VOID NTAPI SmApc() {
    std::string threadId = GetThreadIdString();
    std::cout << "\n*** APC HANDLER EXECUTING - CFG BYPASS SUCCESSFUL ***\n" << std::endl;
    LogMessage("Entered SmApc handler - Thread ID: %s", threadId.c_str());

    // Show MessageBox to demonstrate we're in the APC handler
    std::cout << "Showing MessageBox for APC Handler" << std::endl;
    ShowMessageWithThreadId("SlimBloc APC", "APC Handler Executing\nCFG Bypass Successful!");

    // This function will be called after the thread breaks out of its spin
    // It's responsible for transitioning to SmService2
    SmService2();
}

// Adjust stack based on syscall model and prepare for SmService
VOID SmService2() {
    LogMessage("Entered SmService2 - preparing stack for service dispatch");

    // Get current stack pointer - architecture agnostic approach
    PVOID currentEsp = nullptr;

#ifdef _M_IX86
    // x86 inline assembly
    __asm {
        mov currentEsp, esp
    }
#elif defined(_M_X64)
    // For x64, use intrinsics instead of inline assembly
    currentEsp = _AddressOfReturnAddress();
#else
    #error "Unsupported architecture"
#endif

    LogMessage("Current stack pointer: 0x%p", currentEsp);

    // For demo purposes, we don't actually modify the stack
    // Just call the next stage directly

    // Call SmService which determines what to do next
    SmService();
}

// Main service dispatcher
VOID SmService() {
    std::string threadId = GetThreadIdString();
    std::cout << "\n*** SERVICE HANDLER EXECUTING - ANALYZING CONTEXT ***\n" << std::endl;
    LogMessage("Entered SmService - analyzing execution context - Thread ID: %s", threadId.c_str());

    // Show MessageBox to demonstrate SmService execution
    ShowMessageWithThreadId("SlimBloc Service", "Service Handler Executing\nAnalyzing execution context");

    // Get the return address to determine context
    PVOID returnAddress = _ReturnAddress();
    LogMessage("Return address: 0x%p", returnAddress);

    // Determine which service was called and route accordingly
    if (IsAddressInModule(returnAddress, "ntdll.dll")) {
        // If we were called from NtTestAlert or similar, handle it
        LogMessage("Called from ntdll.dll, treating as CFG bypass path");

        // Check if this matches our VEH calling pattern
        if (g_VehCallingPattern &&
            ((ULONG_PTR)returnAddress >= (ULONG_PTR)g_VehCallingPattern - 0x100) &&
            ((ULONG_PTR)returnAddress <= (ULONG_PTR)g_VehCallingPattern + 0x100)) {
            LogMessage("Detected call from VEH handler area");
        }

        SmXcpt();
    } else {
        // Otherwise we're in a normal execution path
        LogMessage("Standard execution path, returning to caller");
    }
}

// Exception handler that continues execution
VOID SmXcpt() {
    LogMessage("Entered SmXcpt - handling exception flow");

    // This would handle whatever exception caused us to enter this flow
    // For demo, just queue another APC and continue
    SmAlertThread(GetCurrentThread());

    LogMessage("Exception handler complete");
}

// Main worker thread loop
VOID SmWorker() {
    LogMessage("Worker thread entering main loop");

    while (TRUE) {
        //LogMessage("Worker waiting for signal...");

        // Wait for signal from slim lock or work item
        if (NtWaitForAlertByThreadId) {
            // Use direct thread alerting for Windows 10+
            LARGE_INTEGER timeout;
            timeout.QuadPart = -100000; // 10ms in 100ns units (negative = relative time)
            NtWaitForAlertByThreadId((HANDLE)g_WorkerContext.ThreadId, &timeout);
        } else {
            // Use event for older Windows
            WaitForSingleObject(g_WorkerEvent, 10);
        }

        // Lock for thread safety when iterating
        std::lock_guard<std::mutex> lock(g_ThreadContextMutex);

        // Process any threads that need routing
        for (auto& ctx : g_ThreadContexts) {
            if (ctx.SlimLock && ctx.SlimLock->Ptr) {
                LogMessage("Found thread 0x%X in slim lock, routing...", ctx.ThreadId);
                if (SmRouteThread(&ctx)) {
                    LogMessage("Thread successfully routed");
                } else {
                    LogMessage("Failed to route thread");
                }
            }
        }

        // Clean up thread contexts that are no longer needed
        CleanupThreadContexts();
    }
}

// Clean up thread contexts that are no longer needed
VOID CleanupThreadContexts() {
    size_t beforeCount = g_ThreadContexts.size();
    std::vector<THREAD_CONTEXT> contextsToRemove;

    // First identify contexts to remove
    for (auto& ctx : g_ThreadContexts) {
        bool shouldRemove = false;

        // Always check if the thread is still alive first
        if (!IsThreadAlive((HANDLE)(ULONG_PTR)ctx.ThreadId)) {
            LogMessage("Thread 0x%X is no longer alive, marked for cleanup", ctx.ThreadId);
            shouldRemove = true;
        }
        // If SlimLock is NULL, it's already been cleaned up
        else if (!ctx.SlimLock) {
            shouldRemove = true;
        }
        // Check if SlimLock memory is still valid
        else {
            MEMORY_BASIC_INFORMATION memInfo;
            if (VirtualQuery(ctx.SlimLock, &memInfo, sizeof(memInfo)) == 0 ||
                memInfo.State != MEM_COMMIT) {
                LogMessage("Thread 0x%X has invalid SlimLock memory, marked for cleanup", ctx.ThreadId);
                shouldRemove = true;
            }
            // If lock is not active and thread is idle, consider cleanup
            else if (!ctx.SlimLock->Ptr) {
                // Additional check: if thread has been idle for a while
                // For now, we just clean it up immediately if Ptr is not set
                shouldRemove = true;
            }
        }

        if (shouldRemove) {
            contextsToRemove.push_back(ctx);
        }
    }

    // Now safely clean up the identified contexts
    for (auto& ctx : contextsToRemove) {
        // Remove this context from the main vector
        g_ThreadContexts.erase(
            std::remove_if(
                g_ThreadContexts.begin(),
                g_ThreadContexts.end(),
                [&ctx](const THREAD_CONTEXT& item) {
                    return item.ThreadId == ctx.ThreadId;
                }
            ),
            g_ThreadContexts.end()
        );

        // Free the SlimLock memory safely
        if (ctx.SlimLock) {
            MEMORY_BASIC_INFORMATION memInfo;
            if (VirtualQuery(ctx.SlimLock, &memInfo, sizeof(memInfo)) != 0 &&
                memInfo.State == MEM_COMMIT) {
                LogMessage("Freeing SlimLock memory for thread 0x%X", ctx.ThreadId);
                VirtualFree(ctx.SlimLock, 0, MEM_RELEASE);
            }
        }

        // Close the thread handle if we own it
        if (ctx.ThreadHandle) {
            CloseHandle(ctx.ThreadHandle);
        }

        LogMessage("Thread context 0x%X cleaned up", ctx.ThreadId);
    }

    if (contextsToRemove.size() > 0) {
        LogMessage("Cleaned up %d thread contexts, %d remaining",
            contextsToRemove.size(), g_ThreadContexts.size());
    }
}

// Worker thread startup routine
VOID SmStartup() {
    LogMessage("Worker thread starting initialization");

    // Perform initialization activities for the worker thread

    // Use RtlDispatchException to find VEH calling address
    if (RtlDispatchException) {
        LogMessage("Finding VEH calling pattern via exception dispatch");

        __try {
            // Prepare an exception record
            EXCEPTION_RECORD exceptionRecord = {0};
            exceptionRecord.ExceptionCode = STATUS_ACCESS_VIOLATION;
            exceptionRecord.ExceptionFlags = 0;
            exceptionRecord.ExceptionRecord = nullptr;
            exceptionRecord.ExceptionAddress = RtlDispatchException;
            exceptionRecord.NumberParameters = 2;
            exceptionRecord.ExceptionInformation[0] = 0; // write
            exceptionRecord.ExceptionInformation[1] = 0; // address

            // Prepare a context record
            CONTEXT context = {0};
            context.ContextFlags = CONTEXT_FULL;

            // Get current context as starting point
            RtlCaptureContext(&context);

            // Note: In a real implementation we'd use this to trigger ZwTestAlert
            // For the demo we just log the information
            LogMessage("Prepared for exception dispatch");

            // Store info about VEH calling pattern - in real code we'd
            // extract this from the exception handling flow
            g_VehCallingPattern = _ReturnAddress();
            LogMessage("VEH Calling pattern: 0x%p", g_VehCallingPattern);
        }
        __except (EXCEPTION_EXECUTE_HANDLER) {
            LogMessage("Exception handler executed");
        }
    }

    // Enter the worker loop
    SmWorker();
}

// Find the RtlReleaseSRWLockExclusive return address on stack
PVOID FindReturnAddressOnStack(HANDLE ThreadHandle, PVOID StackPtr) {
    if (!ThreadHandle || !StackPtr) return NULL;

    PVOID returnAddresses[20] = {0}; // Array to hold potential return addresses
    SIZE_T bytesRead = 0;

    // Read a chunk of the stack
    if (!ReadProcessMemory(
        GetCurrentProcess(),
        StackPtr,
        returnAddresses,
        sizeof(returnAddresses),
        &bytesRead)) {
        LogMessage("Failed to read thread stack: error %d", GetLastError());
        return NULL;
    }

    // Look for addresses in ntdll.dll
    for (int i = 0; i < (bytesRead / sizeof(PVOID)); i++) {
        if (IsAddressInModule(returnAddresses[i], "ntdll.dll")) {
            LogMessage("Found potential return address in ntdll: 0x%p", returnAddresses[i]);
            return returnAddresses[i];
        }
    }

    return NULL;
}

// Route a thread out of its spin loop
BOOL SmRouteThread(PTHREAD_CONTEXT ThreadCtx) {
    if (!ThreadCtx) {
        LogMessage("Invalid thread context for routing");
        return FALSE;
    }

    // Validate SlimLock before using
    if (!ThreadCtx->SlimLock) {
        LogMessage("ThreadCtx->SlimLock is NULL");
        return FALSE;
    }

    // Check if SlimLock is valid memory
    MEMORY_BASIC_INFORMATION memInfo;
    if (VirtualQuery(ThreadCtx->SlimLock, &memInfo, sizeof(memInfo)) == 0 ||
        memInfo.State != MEM_COMMIT ||
        !(memInfo.Protect & (PAGE_READWRITE | PAGE_EXECUTE_READWRITE))) {
        LogMessage("ThreadCtx->SlimLock is not valid memory: State=%d, Protect=%d",
                   memInfo.State, memInfo.Protect);
        ThreadCtx->SlimLock = NULL; // Mark it as NULL so it will be cleaned up
        return FALSE;
    }

    LogMessage("Routing thread 0x%X", ThreadCtx->ThreadId);

    // Get a handle to the thread if we don't have one
    HANDLE hThread = ThreadCtx->ThreadHandle;
    BOOL needCloseHandle = FALSE;

    if (!hThread) {
        hThread = OpenThread(THREAD_ALL_ACCESS, FALSE, (DWORD)(ULONG_PTR)ThreadCtx->ThreadId);
        if (!hThread) {
            LogMessage("Failed to open thread for routing: error %d", GetLastError());
            return FALSE;
        }
        needCloseHandle = TRUE;
    }

    // Verify the thread is still alive
    if (!IsThreadAlive(hThread)) {
        LogMessage("Thread is no longer alive, cannot route");
        if (needCloseHandle) CloseHandle(hThread);
        return FALSE;
    }

    // Suspend the thread to safely examine its state
    DWORD suspendCount = SuspendThread(hThread);
    if (suspendCount == (DWORD)-1) {
        LogMessage("Failed to suspend thread: error %d", GetLastError());
        if (needCloseHandle) CloseHandle(hThread);
        return FALSE;
    }

    LogMessage("Thread suspended, examining context");

    // Get thread context to find stack pointer
    CONTEXT threadContext = {0};
    threadContext.ContextFlags = CONTEXT_CONTROL | CONTEXT_INTEGER;

    if (!GetThreadContext(hThread, &threadContext)) {
        LogMessage("Failed to get thread context: error %d", GetLastError());
        ResumeThread(hThread);
        if (needCloseHandle) CloseHandle(hThread);
        return FALSE;
    }

    // Find the RtlReleaseSRWLockExclusive return address on stack
    // Use architecture-specific register for stack pointer
#ifdef _M_IX86
    PVOID stackPtr = (PVOID)threadContext.Esp;
#elif defined(_M_X64)
    PVOID stackPtr = (PVOID)threadContext.Rsp;
#else
    #error "Unsupported architecture"
#endif
    LogMessage("Thread stack pointer: 0x%p", stackPtr);

    // Find return address in the stack
    PVOID returnAddress = FindReturnAddressOnStack(hThread, stackPtr);
    if (returnAddress) {
        LogMessage("Found return address: 0x%p", returnAddress);

        // Save original return address
        ThreadCtx->OriginalReturnAddress = returnAddress;

        // Find the stack location with this return address
        PVOID* stackAddresses[50] = {0}; // Array to search
        SIZE_T bytesRead = 0;

        if (ReadProcessMemory(
            GetCurrentProcess(),
            stackPtr,
            stackAddresses,
            sizeof(stackAddresses),
            &bytesRead)) {

            for (size_t i = 0; i < (bytesRead / sizeof(PVOID*)); i++) {
                if (stackAddresses[i] == returnAddress) {
                    // Found the return address on stack, replace with ZwYieldExecution
                    PVOID* addrLocation = (PVOID*)((BYTE*)stackPtr + (i * sizeof(PVOID)));
                    LogMessage("Return address found at stack offset %zu (0x%p)", i, addrLocation);

                    PVOID newReturnAddr = (PVOID)ZwYieldExecution;
                    SIZE_T bytesWritten = 0;

                    if (WriteProcessMemory(
                        GetCurrentProcess(),
                        addrLocation,
                        &newReturnAddr,
                        sizeof(PVOID),
                        &bytesWritten)) {
                        LogMessage("Successfully replaced return address with ZwYieldExecution");
                        ThreadCtx->RouteTarget = newReturnAddr;
                    } else {
                        LogMessage("Failed to write new return address: error %d", GetLastError());
                    }
                    break;
                }
            }
        } else {
            LogMessage("Failed to read stack for return address search: error %d", GetLastError());
        }
    } else {
        LogMessage("Could not find suitable return address on stack");
    }

    // Check again if SlimLock is still valid
    if (VirtualQuery(ThreadCtx->SlimLock, &memInfo, sizeof(memInfo)) == 0 ||
        memInfo.State != MEM_COMMIT) {
        LogMessage("SlimLock became invalid during thread routing");
        ResumeThread(hThread);
        if (needCloseHandle) CloseHandle(hThread);
        return FALSE;
    }

    // Clear the lock pointer to break the spin loop
    __try {
        ThreadCtx->SlimLock->Ptr = NULL;
    } __except(EXCEPTION_EXECUTE_HANDLER) {
        LogMessage("Exception when clearing SlimLock->Ptr");
        ResumeThread(hThread);
        if (needCloseHandle) CloseHandle(hThread);
        return FALSE;
    }

    // Resume the thread to continue execution
    ResumeThread(hThread);
    LogMessage("Thread resumed, should now continue with new return address");

    // Alert the thread if we have the NT alert function
    if (NtAlertThreadByThreadId && ThreadCtx->ThreadId) {
        LogMessage("Sending alert to thread");
        NtAlertThreadByThreadId((HANDLE)ThreadCtx->ThreadId);
    }

    if (needCloseHandle) CloseHandle(hThread);
    return TRUE;
}

// Modify SetupCfgBypass function - remove the local function definition
PVOID SetupCfgBypass() {
    LogMessage("Setting up CFG bypass");

    // Store original function pointer for later restoration
    PVOID originalFunc = (PVOID)NtTestAlert;
    LogMessage("Original function: 0x%p", originalFunc);

    // In our TestAlertWrapper function we'll deliver the APC
    // We don't queue the APC here anymore since that's causing issues
    // Instead, we'll queue it in the test thread function

    LogMessage("CFG bypass will use TestAlertWrapper function to trigger APCs");

    // Return our wrapper function instead
    return (PVOID)TestAlertWrapper;
}

// Modify the TestAlertWrapper function to be more robust
NTSTATUS NTAPI TestAlertWrapper() {
    // Log that we're executing the wrapper
    LogMessage("Executing TestAlertWrapper - delivering APCs");

    __try {
        // Queue an APC to the current thread that will be delivered immediately
        // This ensures the thread has a valid context when the APC runs
        HANDLE currentThread = GetCurrentThread();
        if (currentThread) {
            DWORD threadId = GetCurrentThreadId();
            LogMessage("Queueing APC for current thread ID: 0x%X", threadId);
            SmAlertThread(currentThread);

            // Deliver any pending APCs
            if (NtTestAlert) {
                LogMessage("Calling NtTestAlert to deliver APCs");
                return NtTestAlert();
            }
        }
        else {
            LogMessage("Failed to get current thread handle");
        }
    }
    __except(EXCEPTION_EXECUTE_HANDLER) {
        LogMessage("Exception in TestAlertWrapper: 0x%X", GetExceptionCode());
    }

    return 0;
}

// Helper function to check if a thread is still alive
BOOL IsThreadAlive(HANDLE ThreadHandle) {
    if (!ThreadHandle) return FALSE;

    DWORD exitCode = 0;
    if (GetExitCodeThread(ThreadHandle, &exitCode)) {
        return exitCode == STILL_ACTIVE;
    }
    return FALSE;
}

// Check if an address is within a specified module
BOOL IsAddressInModule(PVOID Address, LPCSTR ModuleName) {
    if (!Address || !ModuleName) return FALSE;

    HMODULE module = GetModuleHandleA(ModuleName);
    if (!module) return FALSE;

    MODULEINFO modInfo = {0};
    if (!GetModuleInformation(GetCurrentProcess(), module, &modInfo, sizeof(modInfo))) {
        return FALSE;
    }

    return ((ULONG_PTR)Address >= (ULONG_PTR)modInfo.lpBaseOfDll &&
            (ULONG_PTR)Address < ((ULONG_PTR)modInfo.lpBaseOfDll + modInfo.SizeOfImage));
}

// Log a formatted message
VOID LogMessage(const char* Format, ...) {
    char buffer[1024] = {0};

    va_list args;
    va_start(args, Format);
    vsprintf_s(buffer, Format, args);
    va_end(args);

    std::string message = "[SlimBloc] ";
    message += buffer;

    OutputDebugStringA(message.c_str());
    std::cout << message << std::endl;
}

// Convert RVA to service ID
BOOL NtRvaToId(PVOID Address, PULONG ServiceId) {
    // This is a simplified version for the demo
    if (!Address || !ServiceId) return FALSE;

    // In a real implementation, this would analyze the syscall instruction
    // to extract the service ID
    *ServiceId = 0;
    return TRUE;
}

std::string GetThreadIdString() {
    std::stringstream ss;
    ss << "0x" << std::hex << GetCurrentThreadId();
    return ss.str();
}

VOID ShowMessageWithThreadId(const char* title, const char* message) {
    std::string threadInfo = "Thread ID: " + GetThreadIdString();
    std::string fullMsg = std::string(message) + "\n" + threadInfo;
    MessageBoxA(NULL, fullMsg.c_str(), title, MB_OK | MB_ICONINFORMATION);
}
